@@ -1,20 +1,20 @@
 ## Load data
-data("compiegne", package = "folio")
+data("iris")
 
 ## Compute principal components analysis
-X <- pca(compiegne, scale = TRUE, sup_col = 7:10)
+X <- pca(iris, scale = TRUE, sup_row = sample(150, 10), sup_col = 2)
 
 ## Get row coordinates
-get_coordinates(X, margin = 1)
+head(get_coordinates(X, margin = 1))
 
 ## Get column coordinates
-get_coordinates(X, margin = 2)
+head(get_coordinates(X, margin = 2))
 
 ## Get row contributions
-get_contributions(X, margin = 1)
+head(get_contributions(X, margin = 1))
 
 ## Get correlations between variables and dimensions
-get_correlations(X)
+head(get_correlations(X))
 
 ## Get eigenvalues
 get_eigenvalues(X)
